/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.filetransfer.service;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.bc.filetransfer.util.FileTransferUtils;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.message.Exchange;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 * 
 * @version
 * @since Petals 2.0
 * @author Adrien LOUIS - EBMWebSourcing
 * 
 */

public class GetFileService extends AbstractGetOperationService {

    public GetFileService(final ConfigurationExtensions extensions, final Logger logger) {
        super(extensions, logger);
    }

    /**
     * Process the exchange : get the file in the configured directory, <br>
     * using filepattern set in the exchange <br>
     * file is returned as OUT response (assume the file is an XML file) <br>
     * return the first file <br>
     * the file is move/deleted after read
     */
    @Override
    public void process(final Exchange exchange) {
        this.logger.log(Level.INFO, "JBI Exchange received - getFile service requested");
        if (exchange.isInOnlyPattern() || exchange.isRobustInOnlyPattern()) {
            exchange
            .setError(new JBIException(
                    "the exchange pattern is not compatible. Operation getFile is InOut or InOptionalOut."));
        } else {
            try {
                final String filePattern = this.extractFilePattern(exchange);
                final File readDirectory = FileTransferUtils.getReadDirectory(this.extensions);
                this.logger.log(Level.INFO, "read files from : " + readDirectory.getAbsolutePath());
                this.logger.log(Level.INFO, "file pattern : " + filePattern);
                final File[] files = this.listFiles(readDirectory, filePattern);
                if (files.length > 0) {
                    final File file = files[0];
                    // check if the file can be read
                    if (!FileTransferUtils.checkAccess(file)) {
                        throw new IOException("File " + file.getName() + " is locked.");
                    }

                    this.logger.log(Level.INFO, "file found : " + file.getName());
                    // move file in a work or backup dir before processing
                    final File processedFile = FileTransferUtils.moveFile(file, this.extensions);
                    // copy file
                    final NormalizedMessage out = exchange.getOutMessage();
                    FileTransferUtils.copyFileToContent(processedFile, out);
                } else {
                    // TODO: The fault should be created from its WSDL description
                    exchange.setFault(new Exception("Unable to find any file for the pattern "
                            + filePattern));
                }
                this.logger.log(Level.INFO, "JBI Exchange received - getFile service requested");
            } catch (final MessagingException e) {
                this.logger.log(Level.SEVERE, e.getMessage(), e);
                exchange.setError(new Exception(e));
            } catch (final IOException e) {
                try {
                    this.logger.warning("can not retrieve file(s). " + e.getClass().getName() + " : "
                            + e.getMessage());
                    this.setIOFaultOnExchange(exchange, e.getMessage());
                } catch (final JBIException e1) {
                    this.logger.log(Level.SEVERE, e1.getMessage(), e1);
                    exchange.setError(e1);
                }
            }
            this.logger.log(Level.INFO, "getFile service is correctly terminated");
        }
    }
}
